/*
 * EdisonServer.c
 * Author: Aaron Needles
 * Copyright (c) 2015 Aaron Needles.
 *
 * Desc:
 * TCP connection, command handler
 * UDP one-way data update, configurable clocking
 * Intel Edison with Arduino breakout board and Grove base shield
 *
 *
 * Issues:
 *   On disconnect from remote .NET client on Windows, the process exits without indicating why.
 *   Putty client disconnect does not have the same issue, and the process drops back to
 *   "Waiting for accept()..."
 *   For Putty use "raw", target IP = <your Edison IP>, Port = 10000.
 *   Once connected, press h for help and q to disconnect.
 *
 *   MRAA library causes WiFi interference.
 *
 */

// For standard stuff
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <string.h>


#include "mraa.h"

// For digital and analog I/O
#include "localIO.h"


#define MAX_BUFFERREAD	400

// size queue actions
#define MAX_ACTIONS		10

// size queue statistic
#define MAX_STATISTIC	10

// time to refresh on remote iot
#define TIME_REFRESH	120 // seconds

// define action tipe
#define ACTION_TURN_ON	1	// action: set lamp on

// system default
#define FALSE 0
#define TRUE 1


// function prototipes
int Serial_Init(int comport);
int WriteSerial(char *data);
int ReadSerial(char *data);

int ReadXbee();
int ParseXbeeMessage(char *Data);
void ManageActions(int Source);
void IncrementStatistic(int Source);

void UpdateStatistic();

// struct for action
typedef struct _action
{
	int Source;			// action source notification
	int Destination;	// action destination node
	int Action;			// action type
	int Delaystart;		// delay action
	int Duration;		// duration action
} ActionEle;

// struct for statistic
typedef struct _statistic
{
	int Source;			// node statistic
	time_t Lastupdate;	// time last update
	int Counter;		// actual counter
} Statistic;

// structures for action
ActionEle ActionsList[MAX_ACTIONS];
Statistic StatisticList[MAX_STATISTIC];

// data for serial communication
mraa_uart_context uart;

// buffer for serial communication
char tempserial[MAX_BUFFERREAD];

int pendingxbeedata = FALSE;
int indexbuffer = 0;

void InitPlannedActions()
{
	// set planned action to do after any notifications

	// action
	// if node 1 notify a transition, the node 2 shall light on for 4000ms after 4000ms
	ActionsList[0].Source = 1;
	ActionsList[0].Action = ACTION_TURN_ON;
	ActionsList[0].Destination = 2;
	ActionsList[0].Delaystart = 4000;
	ActionsList[0].Duration = 4000;

	// action
	// if node 2 notify a transition, the node 1 shall light on for 4000ms after 4000ms
	ActionsList[1].Source = 2;
	ActionsList[1].Action = ACTION_TURN_ON;
	ActionsList[1].Destination = 1;
	ActionsList[1].Delaystart = 4000;
	ActionsList[1].Duration = 4000;
}


// init base
void Init()
{
	// init serial 0 (pin 0 1  on edison)
	Serial_Init(0);

	// reset of all structures
	memset(ActionsList,0,sizeof(ActionsList));
	memset(StatisticList,0,sizeof(StatisticList));

	// set planned action to do after any notifications
	InitPlannedActions();

	// send a message to notifiy all station that the master is on
	WriteSerial("-1 MASTER ON");
}

int main()
{
	// init data
	Init();

	// log
	printf("Init complete\n\n");

	// infinite loop
	while (1) {

		// manage xbee serial
		ReadXbee();

		// if IOT enabled, send data to IOT
#ifdef SENT_IOT_INFO
		UpdateStatistic();
#endif
	}
}

// if IOT enabled, send statistic
#ifdef SENT_IOT_INFO
void UpdateStatistic()
{
	int i;
	// store actual time
	time_t timenow = time(NULL);

	// for all statistic
	for (i = 0 ; i < MAX_STATISTIC; i++)
	{
		// if station present
		if (StatisticList[i].Source != 0)
		{
			// verifiy if time update elapased
			if (timenow - StatisticList[i].Lastupdate >= TIME_REFRESH)
			{
				// TODO: here send statistic to your favourite IoT server

				// reset counter
				StatisticList[i].Counter = 0;

				// update time
				StatisticList[i].Lastupdate = time(NULL);
			}
		}
	}
}
#endif

// init serila communication
int Serial_Init(int comport)
{
	// init base serial
	mraa_init();

	uart = mraa_uart_init(comport);

	if (uart == NULL) {
	        fprintf(stderr, "UART failed to setup\n");
	        return 0;
	}

	//UART Settings
	mraa_uart_set_baudrate(uart, 9600);
	mraa_uart_set_mode(uart, 8,MRAA_UART_PARITY_NONE , 1);
	mraa_uart_set_flowcontrol(uart, 0, 0);
	mraa_uart_set_timeout(uart, 1, 1, 1);

	return 1;
}

// write a termianted string to seriaal
int WriteSerial(char *data)
{
	/**
	 * Write bytes in buffer to a device
	 *
	 * @param dev uart context
	 * @param buf buffer pointer
	 * @param length maximum size of buffer
	 * @return the number of bytes written, or -1 if an error occurred
	 */
	// log
	printf(data);
	printf("\n");

	// send data
	return mraa_uart_write(uart, data, strlen(data));
}

// read data from serial
int ReadSerial(char *data)
{
	// read a single byte
	int result = mraa_uart_read(uart, data, 1);

	// if read ok, terminate string
	if (result > 0)
	{
		data[result] = 0;
	}
	else
	{
		data[0] = 0;
	}

	// return result
	return result;
}

// read and parse messages from xbee
int ReadXbee()
{
	char buffertemp[2];
	char readval;
	int numread;
	int i;

	// while data available
	while( mraa_uart_data_available(uart,1))
	{
		// read 1 byte from serial
		numread = ReadSerial(buffertemp);

		// if something read
		if (numread > 0)
		{
			// for all data read
			for (i = 0; i < numread ; i++)
			{
				// get the byte
				readval = buffertemp[i];

				// if init message
				if (readval == 0x02)
				{
					// reset the buffer and set init message
					indexbuffer = 0;
					pendingxbeedata = TRUE;
				}
				else
				{
					// if pending message ongoing
					if (pendingxbeedata == TRUE)
					{
						// verify if end message
						if (readval == 0x03)
						{
							// terminate the string
							tempserial[indexbuffer] = 0;

							// log
							printf("Xbee --> ");
							printf(tempserial);
							printf("\n");

							// parse the message read
							if (indexbuffer > 0)
							{
								ParseXbeeMessage(tempserial);
							}

							// remove pending message
							pendingxbeedata = FALSE;
						}
						else
						{
							// if not exceed buffer length
							if (indexbuffer < MAX_BUFFERREAD - 1)
							{
								// store the byte in buffer
								tempserial[indexbuffer] = readval;
								indexbuffer++;
							}
							else
							{
								// reset pending message
								pendingxbeedata = FALSE;
								break;
							}
						}
					}
				}
			}
		}
	}

	return 0;
}

// increase the statistic of a node
void IncrementStatistic(int Source)
{
	int i;

	// search for this node in table
	for (i = 0 ; i < MAX_STATISTIC; i++)
	{
		// if node found
		if (StatisticList[i].Source == Source)
		{
			// increment statistic
			StatisticList[i].Counter++;
			break;
		}

		// in found a empty slot, add the node to list
		if (StatisticList[i].Source == 0)
		{
			StatisticList[i].Counter = 1;
			StatisticList[i].Source = Source;
			StatisticList[i].Lastupdate = time(NULL);
			break;
		}
	}
}

// manage actions for each node
void ManageActions(int Source)
{
	int i;
	char mess[MAX_BUFFERREAD];

	// scan the list of actions planned
	for (i = 0 ; i < MAX_ACTIONS; i++)
	{
		// if found an action related to this node notification
		if (ActionsList[i].Source == Source)
		{
			// if the action is a node on
			if (ActionsList[i].Action == ACTION_TURN_ON)
			{
				// by xbee notify the action to do
				sprintf(mess,"\002-1 %d ON %d %d \003",ActionsList[i].Destination,ActionsList[i].Delaystart,ActionsList[i].Duration);

				WriteSerial(mess);
			}
		}
	}
}

// parse messages from xbee
// the messages are: ' x y zzzz' where x is source, y destination, zzzz notify
int ParseXbeeMessage(char *Data)
{
	// log
	printf("Message: %s\n",Data);

	// look for a space
	char *endid = strchr (Data,' ');

	// if found
	if (endid != NULL)
	{
		endid[0] = 0;
		int IdSender = atoi(Data);
		char *endmess = strchr(endid + 1,' ');
		if (endmess != NULL)
		{
			endmess[0] = 0;
			char mess[MAX_BUFFERREAD];
			strcpy(mess,endid + 1);

			// if the notification is a transition of car or other object
			if (strcmp(mess,"PASS:") == 0)
			{
				int CounterPass = atoi(endmess + 1);

				// log
				printf("From slave (%d): new transition (%d)\n",IdSender,CounterPass);

				// send message aknowledge on xbee
				sprintf(mess,"\002-1 %d OK \003",IdSender);
				WriteSerial(mess);

				// manage the related actions
				ManageActions(IdSender);

				// update the statistic for this node
				IncrementStatistic(IdSender);
			}

		}
	}

	return 0;
}



